precision mediump float;

const float PI = 3.14159265;

const float SIGMA = 5.0;     // The SIGMA value for the gaussian function: higher value means more blur
                            // A good value for 9x9 is around 3 to 5
                            // A good value for 7x7 is around 2.5 to 4
                            // A good value for 5x5 is around 2 to 3.5

uniform sampler2D inputImageTexture;
uniform float verticalPass;
uniform float texWidthOrHeight;
uniform float blurFactor;

varying vec2 textureCoordinate;

void main() {
    float blurSize = blurFactor / texWidthOrHeight;

    if (blurSize == 0.) {
      gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
      return;
    }

    float numBlurPixelsPerSide = 6.0;
    vec2 blurMultiplyVec = vec2(1.0 - verticalPass, verticalPass);
    
    // Incremental Gaussian Coefficent Calculation (See GPU Gems 3 pp. 877 - 889)
    vec3 incrementalGaussian;
    incrementalGaussian.x = 1.0 / (sqrt(2.0 * PI) * SIGMA);
    incrementalGaussian.y = exp(-0.5 / (SIGMA * SIGMA));
    incrementalGaussian.z = incrementalGaussian.y * incrementalGaussian.y;
    
    float coefficientSum = 0.0;
    vec4 avgValue = vec4(0.0, 0.0, 0.0, 0.0);
    
    // Take the central sample first...
    avgValue += texture2D(inputImageTexture, textureCoordinate) * incrementalGaussian.x;
    coefficientSum += incrementalGaussian.x;
    incrementalGaussian.xy *= incrementalGaussian.yz;
    
    // Go through the remaining samples
    for (float i = 1.0; i <= numBlurPixelsPerSide; i++) { 
      avgValue += texture2D(inputImageTexture, textureCoordinate - i * blurSize * 
                            blurMultiplyVec) * incrementalGaussian.x;

      avgValue += texture2D(inputImageTexture, textureCoordinate + i * blurSize * 
                            blurMultiplyVec) * incrementalGaussian.x;

      coefficientSum += 2.0 * incrementalGaussian.x;
      incrementalGaussian.xy *= incrementalGaussian.yz;
    }

    gl_FragColor = avgValue / coefficientSum;
}